(ns clj-new.core
  (:gen-class))

(defn -main
  "I don't do a whole lot ... yet."
  [& _]
  (println "I'm a little teapot!"))

(defn dec-maker
  "decrements the argument from the second argument"
  [dec-by]
  #(- % dec-by))

(def dec9 (dec-maker 9))

(defn mapset
  "works like map, but a set"
  [func l]
  (set (map func l))
  )


(def asym-hobbit-body-parts [{:name "head" :size 3}
                             {:name "left-eye" :size 1}
                             {:name "left-ear" :size 1}
                             {:name "mouth" :size 1}
                             {:name "nose" :size 1}
                             {:name "neck" :size 2}
                             {:name "left-shoulder" :size 3}
                             {:name "left-upper-arm" :size 3}
                             {:name "chest" :size 10}
                             {:name "back" :size 10}
                             {:name "left-forearm" :size 3}
                             {:name "abdomen" :size 6}
                             {:name "left-kidney" :size 1}
                             {:name "left-hand" :size 2}
                             {:name "left-knee" :size 2}
                             {:name "left-thigh" :size 4}
                             {:name "left-lower-leg" :size 3}
                             {:name "left-achilles" :size 1}
                             {:name "left-foot" :size 2}])

(defn matching-part
  [part]
  {:name (clojure.string/replace (:name part) #"^left-" "right-")
   :size (:size part)})

(defn symmetrize-body-parts
  "Expects a seq of maps that have a :name and :size"
  [asym-body-parts]
  (reduce (fn [final-body-parts part]
            (into final-body-parts (set [part (matching-part part)])))
          []
          asym-body-parts))

(defn nth-part
  "Creates the nth version of a part starting with \"left\""
  [part n]
  {:name (clojure.string/replace (:name part) #"^left-" (str n "-"))
   :size (:size part)}
  )

(defn alian-symmetrize-body-parts
  "Expects a seq of maps that have a :name and :size"
  [asym-body-parts]
  (reduce (fn [final-body-parts part]
            (into final-body-parts (set [part (nth-part part 2) (nth-part part 3) (nth-part part 4) (nth-part part 5)])))
          []
          asym-body-parts))

(defn arbitrary-symmetrize-body-parts
  "Expects a seq of maps that have a :name and :size, as well as how many of them to make"
  [asym-body-parts num-to-make]
  (reduce (fn [final-body-parts part] (into final-body-parts (set (reduce #(nth-part part %) (range 1 num-to-make))))) [] asym-body-parts)
  )


(defn hit
  [asym-body-parts]
  (let [sym-parts (symmetrize-body-parts asym-body-parts)
        body-part-size-sum (reduce + (map :size sym-parts))
        target (rand body-part-size-sum)]
    (loop [[part & remaining] sym-parts
           accumulated-size (:size part)]
      (if (> accumulated-size target)
        part
        (recur remaining (+ accumulated-size (:size (first remaining))))))))
